﻿using System;
using System.Threading;
using System.Windows.Forms;
using RIOX;

namespace RIOXTestClient
{
    public partial class ClientForm : Form
    {
        private RIOXData _data;
        private RIOXClient _client;

        public ClientForm()
        {
            InitializeComponent();            
        }

        /// <summary>
        /// Connect to host
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                _data = new RIOXData();
                _client = new RIOXClient(typeof(RIOXData), "localhost", 1234);
                _client.ObjectReceivedEvent += ClientObjectReceivedEvent;
                _client.ServerDisconnectedEvent += new EventHandler(_client_ServerDisconnectedEvent);
                // allow time to connect and settle
                Thread.Sleep(1000);
                //Set the Update type to Pushed with timed updates every 5 seconds.
                _client.SendCommand(new RIOXCommand("UpDateType", "PSH:1000"));
                Thread.Sleep(200);
                // Subscribe to CAT commands needed (no spaces between commands)
                _client.SendCommand(new RIOXCommand("Sub", "ZZFA;ZZFB;ZZPC;ZZMD;ZZME;"));

            }

            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, ex.StackTrace);
            }
        }
        void _client_ServerDisconnectedEvent (object sender, EventArgs e)
        {
            MessageBox.Show(new Form() { TopMost = true },
                "The DDUtil TCP Server has disconnected.\r\r" + 
            "You have the options of trying to re-connect DDUtil or\r" + 
            "to terminate this program and take a nap.\r\r" +
            "If you reconnect DDUtil, you will need to press the 'Connect' and 'Subscribe'\r" +
            "buttons again to re-establish communication with the server.",
            "TCP Server Disconnect!",
            MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        void ClientObjectReceivedEvent(object o, RIOXClient.ObjectReceivedEventArgs e)
        {
            _data = (RIOXData)e.DataObject;
      //      Console.WriteLine("Object received: " + _radio);
            UpdateForm();
        }

        void UpdateForm()
        {
            try
            {
                if (InvokeRequired)
                {
                    Invoke((MethodInvoker)delegate
                   {
                       if (_data.ContainsKey("ZZAC"))
                           txtTunStep.Text = (String)_data["ZZAC"];
                       if (_data.ContainsKey("ZZAG"))
                           txtAudio.Text = (String)_data["ZZAG"];
                       if (_data.ContainsKey("ZZBS"))
                           txtBandR1.Text = (String)_data["ZZBS"];
                       if (_data.ContainsKey("ZZBT"))
                           txtBandR2.Text = (String)_data["ZZBT"];
                       if (_data.ContainsKey("ZZBU"))
                           txtCw.Text = (String)_data["ZZBU"];
                       if (_data.ContainsKey("ZZCS"))
                           txtCw.Text = (String)_data["ZZCS"];
                       if (_data.ContainsKey("ZZCU"))
                           txtCpu.Text = (String)_data["ZZCU"];
                       if (_data.ContainsKey("ZZDM"))
                           txtDispMode.Text = (String)_data["ZZDM"];
                       if (_data.ContainsKey("ZZFA"))
                           txtVFOA.Text = (String)_data["ZZFA"];
                       if (_data.ContainsKey("ZZFB"))
                           txtVFOB.Text = (String)_data["ZZFB"];
                       if (_data.ContainsKey("ZZFI"))
                           txtFltR1.Text = (String)_data["ZZFI"];
                       if (_data.ContainsKey("ZZFJ"))
                           txtFltR2.Text = (String)_data["ZZFJ"];
                       if (_data.ContainsKey("ZZGT"))
                           txtAgc.Text = (String)_data["ZZGT"];
                       if (_data.ContainsKey("ZZKS"))
                           txtCwx.Text = (String)_data["ZZKS"];
                       if (_data.ContainsKey("ZZMD"))
                           txtModeR1.Text = (String)_data["ZZMD"];
                       if (_data.ContainsKey("ZZME"))
                           txtModeR2.Text = (String)_data["ZZME"];
                       if (_data.ContainsKey("ZZOA"))
                           txtRx1.Text = (String)_data["ZZOA"];
                       if (_data.ContainsKey("ZZOB"))
                           txtRx2.Text = (String)_data["ZZOB"];
                       if (_data.ContainsKey("ZZOC"))
                           txtTxAnt.Text = (String)_data["ZZOC"];
                       if (_data.ContainsKey("ZZOF"))
                           txtTxLines.Text = (String)_data["ZZOF"];
                       if (_data.ContainsKey("ZZPA"))
                           txtPreamp.Text = (String)_data["ZZPA"];
                       if (_data.ContainsKey("ZZPC"))
                           txtDrive.Text = (String)_data["ZZPC"];
                       if (_data.ContainsKey("ZZRF"))
                           txtRitFreq.Text = (String)_data["ZZRF"];
                       if (_data.ContainsKey("ZZRV"))
                           txtVolts.Text = (String)_data["ZZRV"];
                       if (_data.ContainsKey("ZZSM"))
                           txtSmtr.Text = (String)_data["ZZSM"];
                       if (_data.ContainsKey("ZZSW"))
                           txtTxVfo.Text = (String)_data["ZZSW"];
                       if (_data.ContainsKey("ZZTO"))
                           txtDrive.Text = (String)_data["ZZTO"];
                       if (_data.ContainsKey("ZZTS"))
                           txtTemp.Text = (String)_data["ZZTS"];
                       if (_data.ContainsKey("ZZXF"))
                           txtXitFreq.Text = (String)_data["ZZXF"];
                       // Bool Vars
                       if (_data.ContainsKey("ZZMU"))
                           chkMulRxSts.Checked = (String)_data["ZZMU"] == "1" ? true : false;
                       if (_data.ContainsKey("ZZRS"))
                           chkRx2Sts.Checked = (String)_data["ZZRS"] == "1" ? true : false;
                       if (_data.ContainsKey("ZZRT"))
                           chkRitSts.Checked = (String)_data["ZZRT"] == "1" ? true : false;
                       if (_data.ContainsKey("ZZSP"))
                           chkSplit.Checked = (String)_data["ZZSP"] == "1" ? true : false;
                       if (_data.ContainsKey("ZZTU"))
                           chkTune.Checked = (String)_data["ZZTU"] == "1" ? true : false;
                       if (_data.ContainsKey("ZZTX"))
                           chkMox.Checked = (String)_data["ZZTX"] == "1" ? true : false;
                       if (_data.ContainsKey("ZZXS"))
                           chkXitSts.Checked = (String)_data["ZZXS"] == "1" ? true : false;

                   });
                }
                else
                {
                    if (_data.ContainsKey("ZZAC"))
                        txtTunStep.Text = (String)_data["ZZAC"];
                    if (_data.ContainsKey("ZZAG"))
                        txtAudio.Text = (String)_data["ZZAG"];
                    if (_data.ContainsKey("ZZBS"))
                        txtBandR1.Text = (String)_data["ZZBS"];
                    if (_data.ContainsKey("ZZBT"))
                        txtBandR2.Text = (String)_data["ZZBT"];
                    if (_data.ContainsKey("ZZBU"))
                        txtCw.Text = (String)_data["ZZBU"];
                    if (_data.ContainsKey("ZZCS"))
                        txtCw.Text = (String)_data["ZZCS"];
                    if (_data.ContainsKey("ZZCU"))
                        txtCpu.Text = (String)_data["ZZCU"];
                    if (_data.ContainsKey("ZZDM"))
                        txtDispMode.Text = (String)_data["ZZDM"];
                    if (_data.ContainsKey("ZZFA"))
                        txtVFOA.Text = (String)_data["ZZFA"];
                    if (_data.ContainsKey("ZZFB"))
                        txtVFOB.Text = (String)_data["ZZFB"];
                    if (_data.ContainsKey("ZZFI"))
                        txtFltR1.Text = (String)_data["ZZFI"];
                    if (_data.ContainsKey("ZZFJ"))
                        txtFltR2.Text = (String)_data["ZZFJ"];
                    if (_data.ContainsKey("ZZGT"))
                        txtAgc.Text = (String)_data["ZZGT"];
                    if (_data.ContainsKey("ZZKS"))
                        txtCwx.Text = (String)_data["ZZKS"];
                    if (_data.ContainsKey("ZZMD"))
                        txtModeR1.Text = (String)_data["ZZMD"];
                    if (_data.ContainsKey("ZZME"))
                        txtModeR2.Text = (String)_data["ZZME"];
                    if (_data.ContainsKey("ZZOA"))
                        txtRx1.Text = (String)_data["ZZOA"];
                    if (_data.ContainsKey("ZZOB"))
                        txtRx2.Text = (String)_data["ZZOB"];
                    if (_data.ContainsKey("ZZOC"))
                        txtTxAnt.Text = (String)_data["ZZOC"];
                    if (_data.ContainsKey("ZZOF"))
                        txtTxLines.Text = (String)_data["ZZOF"];
                    if (_data.ContainsKey("ZZPA"))
                        txtPreamp.Text = (String)_data["ZZPA"];
                    if (_data.ContainsKey("ZZPC"))
                        txtDrive.Text = (String)_data["ZZPC"];
                    if (_data.ContainsKey("ZZRF"))
                        txtRitFreq.Text = (String)_data["ZZRF"];
                    if (_data.ContainsKey("ZZRV"))
                        txtVolts.Text = (String)_data["ZZRV"];
                    if (_data.ContainsKey("ZZSM"))
                        txtSmtr.Text = (String)_data["ZZSM"];
                    if (_data.ContainsKey("ZZSW"))
                        txtTxVfo.Text = (String)_data["ZZSW"];
                    if (_data.ContainsKey("ZZTO"))
                        txtDrive.Text = (String)_data["ZZTO"];
                    if (_data.ContainsKey("ZZTS"))
                        txtTemp.Text = (String)_data["ZZTS"];
                    if (_data.ContainsKey("ZZXF"))
                        txtXitFreq.Text = (String)_data["ZZXF"];
                    // Bool Vars
                    if (_data.ContainsKey("ZZMU"))
                        chkMulRxSts.Checked = (String)_data["ZZMU"] == "1" ? true : false;
                    if (_data.ContainsKey("ZZRS"))
                        chkRx2Sts.Checked = (String)_data["ZZRS"] == "1" ? true : false;
                    if (_data.ContainsKey("ZZRT"))
                        chkRitSts.Checked = (String)_data["ZZRT"] == "1" ? true : false;
                    if (_data.ContainsKey("ZZSP"))
                        chkSplit.Checked = (String)_data["ZZSP"] == "1" ? true : false;
                    if (_data.ContainsKey("ZZTU"))
                        chkTune.Checked = (String)_data["ZZTU"] == "1" ? true : false;
                    if (_data.ContainsKey("ZZTX"))
                        chkMox.Checked = (String)_data["ZZTX"] == "1" ? true : false;
                    if (_data.ContainsKey("ZZXS"))
                        chkXitSts.Checked = (String)_data["ZZXS"] == "1" ? true : false;
                }
            }
            catch { }
        }

        private void ClientForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (_client != null)
            {
                _client.SendCommand(new RIOXCommand("UnSub", "none"));
                _client.Close();
            }
        }

        private void btnCmd_Click(object sender, EventArgs e)
        {
            _client.SendCommand(new RIOXCommand("ClientCmd", txtCmd2Send.Text)); 
        }

        /// <summary>
        /// Subscribe CAT commands to the host. Commmands may be single or mixed.
        /// Must use this format("Sub", "ZZFA;ZZFB;")
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSub_Click(object sender, EventArgs e)
        {
            _client.SendCommand(new RIOXCommand("Sub", txtSub.Text));
        }

        /// <summary>
        /// Un-subscribe all commands from the host
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUnSub_Click(object sender, EventArgs e)
        {
            _client.SendCommand(new RIOXCommand("UnSub", "none"));
        }

        /// <summary>
        /// Sets the RIOX Server data update type
        /// i.e. _client.SendCommand(new RIOXCommand("UpDateType","INT:30000"));
        /// sends updates to all vars every 30 seconds.
        /// i.e. _client.SendCommand(new RIOXCommand("UpDateType","RAN:60000"));
        /// sends updates when they happen and all every 1 minutes.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnIntrvl_Click (object sender, EventArgs e)
        {
            _client.SendCommand(new RIOXCommand("UpDateType",txtUpDateType.Text));
        }

    }
}
